"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.PluginApi = void 0;

const type_1 = require("@spark-build/web-utils/lib/type");

class PluginApi {
  constructor(id, server) {
    this.server = server;
    this.server.plugin.register(id, this);
    this.id = id;
  }

  describe({
    id,
    enable,
    onReCompile
  } = {}) {
    if (id && this.id !== id) {
      this.id = id;
    }

    const enableBy = enable !== null && enable !== void 0 ? enable : () => false;

    this.enable = () => {
      const result = enableBy();
      this.skipOrRemoveSkip(result);
      return result;
    };

    if (onReCompile) {
      this.onReCompile = () => {
        var _this$enable, _this$enable2;

        return onReCompile((_this$enable = (_this$enable2 = this.enable) === null || _this$enable2 === void 0 ? void 0 : _this$enable2.call(this)) !== null && _this$enable !== void 0 ? _this$enable : false);
      };

      this.onReCompile();
    } else {
      this.enable();
    }
  }

  register(id, plugin) {
    this.server.plugin.register(id, plugin);
    return this;
  }

  addHook(hook) {
    this.server.hooks.add({ ...hook,
      pluginId: this.id
    });
  }

  skip() {
    this.server.plugin.skip(this.id);
  }

  removeSkip() {
    this.server.plugin.removeSkip(this.id);
  }

  skipOrRemoveSkip(enable) {
    const method = enable ? 'removeSkip' : 'skip';
    return this[method]();
  }

  registerMethod({
    name,
    fn,
    exitsError = true
  }) {
    if (this.server.methods.has(name)) {
      if (exitsError) {
        throw new Error(`api.registerMethod() failed, method ${name} is already exist.`);
      } else {
        return;
      }
    }

    const realFn = fn || function (f) {
      const hook = {
        key: name,
        ...(type_1.isObj(f) ? f : {
          fn: f
        })
      };
      this.addHook(hook);
    };

    this.server.methods.set(name, realFn);
  }

}

exports.PluginApi = PluginApi;